import * as tslib_1 from "tslib";
import { isPrimitive, isThenable } from '@sentry/utils/is';
import { logger } from '@sentry/utils/logger';
import { uuid4 } from '@sentry/utils/misc';
import { truncate } from '@sentry/utils/string';
import { SyncPromise } from '@sentry/utils/syncpromise';
import { Dsn } from './dsn';
import { setupIntegrations } from './integration';
/**
 * Base implementation for all JavaScript SDK clients.
 *
 * Call the constructor with the corresponding backend constructor and options
 * specific to the client subclass. To access these options later, use
 * {@link Client.getOptions}. Also, the Backend instance is available via
 * {@link Client.getBackend}.
 *
 * If a Dsn is specified in the options, it will be parsed and stored. Use
 * {@link Client.getDsn} to retrieve the Dsn at any moment. In case the Dsn is
 * invalid, the constructor will throw a {@link SentryException}. Note that
 * without a valid Dsn, the SDK will not send any events to Sentry.
 *
 * Before sending an event via the backend, it is passed through
 * {@link BaseClient.prepareEvent} to add SDK information and scope data
 * (breadcrumbs and context). To add more custom information, override this
 * method and extend the resulting prepared event.
 *
 * To issue automatically created events (e.g. via instrumentation), use
 * {@link Client.captureEvent}. It will prepare the event and pass it through
 * the callback lifecycle. To issue auto-breadcrumbs, use
 * {@link Client.addBreadcrumb}.
 *
 * @example
 * class NodeClient extends BaseClient<NodeBackend, NodeOptions> {
 *   public constructor(options: NodeOptions) {
 *     super(NodeBackend, options);
 *   }
 *
 *   // ...
 * }
 */
export class BaseClient {
    /**
     * Initializes this client instance.
     *
     * @param backendClass A constructor function to create the backend.
     * @param options Options for the client.
     */
    constructor(backendClass, options) {
        /** Is the client still processing a call? */
        this._processing = false;
        this._backend = new backendClass(options);
        this._options = options;
        if (options.dsn) {
            this._dsn = new Dsn(options.dsn);
        }
        this._integrations = setupIntegrations(this._options);
    }
    /**
     * @inheritDoc
     */
    captureException(exception, hint, scope) {
        let eventId = hint && hint.event_id;
        this._processing = true;
        this._getBackend()
            .eventFromException(exception, hint)
            .then(event => this._processEvent(event, hint, scope))
            .then(finalEvent => {
            // We need to check for finalEvent in case beforeSend returned null
            eventId = finalEvent && finalEvent.event_id;
            this._processing = false;
        })
            .catch(reason => {
            logger.log(reason);
            this._processing = false;
        });
        return eventId;
    }
    /**
     * @inheritDoc
     */
    captureMessage(message, level, hint, scope) {
        let eventId = hint && hint.event_id;
        this._processing = true;
        const promisedEvent = isPrimitive(message)
            ? this._getBackend().eventFromMessage(`${message}`, level, hint)
            : this._getBackend().eventFromException(message, hint);
        promisedEvent
            .then(event => this._processEvent(event, hint, scope))
            .then(finalEvent => {
            // We need to check for finalEvent in case beforeSend returned null
            eventId = finalEvent && finalEvent.event_id;
            this._processing = false;
        })
            .catch(reason => {
            logger.log(reason);
            this._processing = false;
        });
        return eventId;
    }
    /**
     * @inheritDoc
     */
    captureEvent(event, hint, scope) {
        let eventId = hint && hint.event_id;
        this._processing = true;
        this._processEvent(event, hint, scope)
            .then(finalEvent => {
            // We need to check for finalEvent in case beforeSend returned null
            eventId = finalEvent && finalEvent.event_id;
            this._processing = false;
        })
            .catch(reason => {
            logger.log(reason);
            this._processing = false;
        });
        return eventId;
    }
    /**
     * @inheritDoc
     */
    getDsn() {
        return this._dsn;
    }
    /**
     * @inheritDoc
     */
    getOptions() {
        return this._options;
    }
    /**
     * @inheritDoc
     */
    flush(timeout) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return (yield Promise.all([
                this._getBackend()
                    .getTransport()
                    .close(timeout),
                this._isClientProcessing(),
            ])).reduce((prev, current) => prev && current);
        });
    }
    /**
     * @inheritDoc
     */
    close(timeout) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return this.flush(timeout).finally(() => {
                this.getOptions().enabled = false;
            });
        });
    }
    /**
     * @inheritDoc
     */
    getIntegrations() {
        return this._integrations || {};
    }
    /**
     * @inheritDoc
     */
    getIntegration(integration) {
        try {
            return this._integrations[integration.id] || null;
        }
        catch (_oO) {
            logger.warn(`Cannot retrieve integration ${integration.id} from the current Client`);
            return null;
        }
    }
    /** Waits for the client to be done with processing. */
    _isClientProcessing(counter = 0) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                if (this._processing) {
                    // Safeguard in case of endless recursion
                    if (counter >= 10) {
                        resolve(false);
                    }
                    else {
                        setTimeout(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
                            resolve(yield this._isClientProcessing(counter + 1));
                        }), 10);
                    }
                }
                else {
                    resolve(true);
                }
            });
        });
    }
    /** Returns the current backend. */
    _getBackend() {
        return this._backend;
    }
    /** Determines whether this SDK is enabled and a valid Dsn is present. */
    _isEnabled() {
        return this.getOptions().enabled !== false && this._dsn !== undefined;
    }
    /**
     * Adds common information to events.
     *
     * The information includes release and environment from `options`,
     * breadcrumbs and context (extra, tags and user) from the scope.
     *
     * Information that is already present in the event is never overwritten. For
     * nested objects, such as the context, keys are merged.
     *
     * @param event The original event.
     * @param hint May contain additional informartion about the original exception.
     * @param scope A scope containing event metadata.
     * @returns A new event with more information.
     */
    _prepareEvent(event, scope, hint) {
        const { environment, release, dist, maxValueLength = 250 } = this.getOptions();
        const prepared = Object.assign({}, event);
        if (prepared.environment === undefined && environment !== undefined) {
            prepared.environment = environment;
        }
        if (prepared.release === undefined && release !== undefined) {
            prepared.release = release;
        }
        if (prepared.dist === undefined && dist !== undefined) {
            prepared.dist = dist;
        }
        if (prepared.message) {
            prepared.message = truncate(prepared.message, maxValueLength);
        }
        const exception = prepared.exception && prepared.exception.values && prepared.exception.values[0];
        if (exception && exception.value) {
            exception.value = truncate(exception.value, maxValueLength);
        }
        const request = prepared.request;
        if (request && request.url) {
            request.url = truncate(request.url, maxValueLength);
        }
        if (prepared.event_id === undefined) {
            prepared.event_id = uuid4();
        }
        // We prepare the result here with a resolved Event.
        let result = SyncPromise.resolve(prepared);
        // This should be the last thing called, since we want that
        // {@link Hub.addEventProcessor} gets the finished prepared event.
        if (scope) {
            // In case we have a hub we reassign it.
            result = scope.applyToEvent(prepared, hint);
        }
        return result;
    }
    /**
     * Processes an event (either error or message) and sends it to Sentry.
     *
     * This also adds breadcrumbs and context information to the event. However,
     * platform specific meta data (such as the User's IP address) must be added
     * by the SDK implementor.
     *
     *
     * @param event The event to send to Sentry.
     * @param hint May contain additional informartion about the original exception.
     * @param scope A scope containing event metadata.
     * @returns A SyncPromise that resolves with the event or rejects in case event was/will not be send.
     */
    _processEvent(event, hint, scope) {
        const { beforeSend, sampleRate } = this.getOptions();
        if (!this._isEnabled()) {
            return SyncPromise.reject('SDK not enabled, will not send event.');
        }
        // 1.0 === 100% events are sent
        // 0.0 === 0% events are sent
        if (typeof sampleRate === 'number' && Math.random() > sampleRate) {
            return SyncPromise.reject('This event has been sampled, will not send event.');
        }
        return new SyncPromise((resolve, reject) => {
            this._prepareEvent(event, scope, hint).then(prepared => {
                if (prepared === null) {
                    reject('An event processor returned null, will not send event.');
                    return;
                }
                let finalEvent = prepared;
                try {
                    const isInternalException = hint && hint.data && hint.data.__sentry__ === true;
                    if (isInternalException || !beforeSend) {
                        this._getBackend().sendEvent(finalEvent);
                        resolve(finalEvent);
                        return;
                    }
                    const beforeSendResult = beforeSend(prepared, hint);
                    if (typeof beforeSendResult === 'undefined') {
                        logger.error('`beforeSend` method has to return `null` or a valid event.');
                    }
                    else if (isThenable(beforeSendResult)) {
                        this._handleAsyncBeforeSend(beforeSendResult, resolve, reject);
                    }
                    else {
                        finalEvent = beforeSendResult;
                        if (finalEvent === null) {
                            logger.log('`beforeSend` returned `null`, will not send event.');
                            resolve(null);
                            return;
                        }
                        // From here on we are really async
                        this._getBackend().sendEvent(finalEvent);
                        resolve(finalEvent);
                    }
                }
                catch (exception) {
                    this.captureException(exception, {
                        data: {
                            __sentry__: true,
                        },
                        originalException: exception,
                    });
                    reject('`beforeSend` throw an error, will not send event.');
                }
            });
        });
    }
    /**
     * Resolves before send Promise and calls resolve/reject on parent SyncPromise.
     */
    _handleAsyncBeforeSend(beforeSend, resolve, reject) {
        beforeSend
            .then(processedEvent => {
            if (processedEvent === null) {
                reject('`beforeSend` returned `null`, will not send event.');
                return;
            }
            // From here on we are really async
            this._getBackend().sendEvent(processedEvent);
            resolve(processedEvent);
        })
            .catch(e => {
            reject(`beforeSend rejected with ${e}`);
        });
    }
}
//# sourceMappingURL=baseclient.js.map