import { SentryError } from '@sentry/utils/error';
/** Regular expression used to parse a Dsn. */
const DSN_REGEX = /^(?:(\w+):)\/\/(?:(\w+)(?::(\w+))?@)([\w\.-]+)(?::(\d+))?\/(.+)/;
/** Error message */
const ERROR_MESSAGE = 'Invalid Dsn';
/** The Sentry Dsn, identifying a Sentry instance and project. */
export class Dsn {
    /** Creates a new Dsn component */
    constructor(from) {
        if (typeof from === 'string') {
            this._fromString(from);
        }
        else {
            this._fromComponents(from);
        }
        this._validate();
    }
    /**
     * Renders the string representation of this Dsn.
     *
     * By default, this will render the public representation without the password
     * component. To get the deprecated private _representation, set `withPassword`
     * to true.
     *
     * @param withPassword When set to true, the password will be included.
     */
    toString(withPassword = false) {
        // tslint:disable-next-line:no-this-assignment
        const { host, path, pass, port, projectId, protocol, user } = this;
        return (`${protocol}://${user}${withPassword && pass ? `:${pass}` : ''}` +
            `@${host}${port ? `:${port}` : ''}/${path ? `${path}/` : path}${projectId}`);
    }
    /** Parses a string into this Dsn. */
    _fromString(str) {
        const match = DSN_REGEX.exec(str);
        if (!match) {
            throw new SentryError(ERROR_MESSAGE);
        }
        const [protocol, user, pass = '', host, port = '', lastPath] = match.slice(1);
        let path = '';
        let projectId = lastPath;
        const split = projectId.split('/');
        if (split.length > 1) {
            path = split.slice(0, -1).join('/');
            projectId = split.pop();
        }
        Object.assign(this, { host, pass, path, projectId, port, protocol, user });
    }
    /** Maps Dsn components into this instance. */
    _fromComponents(components) {
        this.protocol = components.protocol;
        this.user = components.user;
        this.pass = components.pass || '';
        this.host = components.host;
        this.port = components.port || '';
        this.path = components.path || '';
        this.projectId = components.projectId;
    }
    /** Validates this Dsn and throws on error. */
    _validate() {
        ['protocol', 'user', 'host', 'projectId'].forEach(component => {
            if (!this[component]) {
                throw new SentryError(ERROR_MESSAGE);
            }
        });
        if (this.protocol !== 'http' && this.protocol !== 'https') {
            throw new SentryError(ERROR_MESSAGE);
        }
        if (this.port && Number.isNaN(parseInt(this.port, 10))) {
            throw new SentryError(ERROR_MESSAGE);
        }
    }
}
//# sourceMappingURL=dsn.js.map