import { addGlobalEventProcessor, getCurrentHub } from '@sentry/hub';
import { isRegExp } from '@sentry/utils/is';
import { logger } from '@sentry/utils/logger';
import { getEventDescription } from '@sentry/utils/misc';
// "Script error." is hard coded into browsers for errors that it can't read.
// this is the result of a script being pulled in from an external domain and CORS.
const DEFAULT_IGNORE_ERRORS = [/^Script error\.?$/, /^Javascript error: Script error\.? on line 0$/];
/** Inbound filters configurable by the user */
export class InboundFilters {
    constructor(_options = {}) {
        this._options = _options;
        /**
         * @inheritDoc
         */
        this.name = InboundFilters.id;
    }
    /**
     * @inheritDoc
     */
    setupOnce() {
        addGlobalEventProcessor((event) => {
            const hub = getCurrentHub();
            if (!hub) {
                return event;
            }
            const self = hub.getIntegration(InboundFilters);
            if (self) {
                const client = hub.getClient();
                const clientOptions = client ? client.getOptions() : {};
                const options = self._mergeOptions(clientOptions);
                if (self._shouldDropEvent(event, options)) {
                    return null;
                }
            }
            return event;
        });
    }
    /** JSDoc */
    _shouldDropEvent(event, options) {
        if (this._isSentryError(event, options)) {
            logger.warn(`Event dropped due to being internal Sentry Error.\nEvent: ${getEventDescription(event)}`);
            return true;
        }
        if (this._isIgnoredError(event, options)) {
            logger.warn(`Event dropped due to being matched by \`ignoreErrors\` option.\nEvent: ${getEventDescription(event)}`);
            return true;
        }
        if (this._isBlacklistedUrl(event, options)) {
            logger.warn(`Event dropped due to being matched by \`blacklistUrls\` option.\nEvent: ${getEventDescription(event)}.\nUrl: ${this._getEventFilterUrl(event)}`);
            return true;
        }
        if (!this._isWhitelistedUrl(event, options)) {
            logger.warn(`Event dropped due to not being matched by \`whitelistUrls\` option.\nEvent: ${getEventDescription(event)}.\nUrl: ${this._getEventFilterUrl(event)}`);
            return true;
        }
        return false;
    }
    /** JSDoc */
    _isSentryError(event, options = {}) {
        if (!options.ignoreInternal) {
            return false;
        }
        try {
            // tslint:disable-next-line:no-unsafe-any
            return event.exception.values[0].type === 'SentryError';
        }
        catch (_oO) {
            return false;
        }
    }
    /** JSDoc */
    _isIgnoredError(event, options = {}) {
        if (!options.ignoreErrors || !options.ignoreErrors.length) {
            return false;
        }
        return this._getPossibleEventMessages(event).some(message => 
        // Not sure why TypeScript complains here...
        options.ignoreErrors.some(pattern => this._isMatchingPattern(message, pattern)));
    }
    /** JSDoc */
    _isBlacklistedUrl(event, options = {}) {
        // TODO: Use Glob instead?
        if (!options.blacklistUrls || !options.blacklistUrls.length) {
            return false;
        }
        const url = this._getEventFilterUrl(event);
        return !url ? false : options.blacklistUrls.some(pattern => this._isMatchingPattern(url, pattern));
    }
    /** JSDoc */
    _isWhitelistedUrl(event, options = {}) {
        // TODO: Use Glob instead?
        if (!options.whitelistUrls || !options.whitelistUrls.length) {
            return true;
        }
        const url = this._getEventFilterUrl(event);
        return !url ? true : options.whitelistUrls.some(pattern => this._isMatchingPattern(url, pattern));
    }
    /** JSDoc */
    _mergeOptions(clientOptions = {}) {
        return {
            blacklistUrls: [...(this._options.blacklistUrls || []), ...(clientOptions.blacklistUrls || [])],
            ignoreErrors: [
                ...(this._options.ignoreErrors || []),
                ...(clientOptions.ignoreErrors || []),
                ...DEFAULT_IGNORE_ERRORS,
            ],
            ignoreInternal: typeof this._options.ignoreInternal !== 'undefined' ? this._options.ignoreInternal : true,
            whitelistUrls: [...(this._options.whitelistUrls || []), ...(clientOptions.whitelistUrls || [])],
        };
    }
    /** JSDoc */
    _isMatchingPattern(value, pattern) {
        if (isRegExp(pattern)) {
            return pattern.test(value);
        }
        if (typeof pattern === 'string') {
            return value.includes(pattern);
        }
        return false;
    }
    /** JSDoc */
    _getPossibleEventMessages(event) {
        if (event.message) {
            return [event.message];
        }
        if (event.exception) {
            try {
                // tslint:disable-next-line:no-unsafe-any
                const { type, value } = event.exception.values[0];
                return [`${value}`, `${type}: ${value}`];
            }
            catch (oO) {
                logger.error(`Cannot extract message for event ${getEventDescription(event)}`);
                return [];
            }
        }
        return [];
    }
    /** JSDoc */
    _getEventFilterUrl(event) {
        try {
            if (event.stacktrace) {
                // tslint:disable:no-unsafe-any
                const frames = event.stacktrace.frames;
                return frames[frames.length - 1].filename;
            }
            if (event.exception) {
                // tslint:disable:no-unsafe-any
                const frames = event.exception.values[0].stacktrace.frames;
                return frames[frames.length - 1].filename;
            }
            return null;
        }
        catch (oO) {
            logger.error(`Cannot extract url for event ${getEventDescription(event)}`);
            return null;
        }
    }
}
/**
 * @inheritDoc
 */
InboundFilters.id = 'InboundFilters';
//# sourceMappingURL=inboundfilters.js.map